--[[
A Bartender4-specific version of ThereIsNoCost by L'ombra:
http://wow.curse.com/downloads/wow-addons/details/thereisnocost.aspx

Credit for idea and original implementation goes there.

TODO:  recheck all this in the presence of macros with items
]]

local LAB = LibStub and LibStub("LibActionButton-1.0",true)
if not LAB then return end

local GetCVar = GetCVar
local SetCVar = SetCVar
local IsConsumableAction = IsConsumableAction
local IsStackableAction = IsStackableAction
local GetActionCount = GetActionCount
local GetActionInfo = GetActionInfo
local GetMacroItem = GetMacroItem
local GetActionText = GetActionText
local pairs = pairs

local tooltip = CreateFrame("GameTooltip", "BT4ThereIsNoCostTooltip", nil, "GameTooltipTemplate")
local eventframe = CreateFrame("Frame")
local checked_actions = {}
local action2button = {}  -- kludge

local CHARGE = ITEM_SPELL_CHARGES:gsub("%%d%s+|4(.+):.*;", "^%%d+ %1$") -- Charge
local CHARGES = ITEM_SPELL_CHARGES:gsub("%%d%s+|4.+:(.*);", "^%%d+ %1$") -- Charges

-- Based on the action currently mapped to BUTTON, figure out whether reagent
-- count text should be suppressed.  Returns true if it should be.  The result
-- is cached because tooltip scanning is expensive.
local function lookup_action (button)
	local count = button.count
	if not count then return end -- how in the hell...
	local action = button._state_action

	local isConStack = not not button:IsConsumableOrStackable()
	if (not isConStack) and not (count:GetText() and count:GetText() > 0) then return end
--safeprint("checking button ID", button.id, "vs action ID", action, isConStack, button)

	-- Annoyingly, when the buttons are created the first time, glyphs have not yet
	-- taken effect, so reagents are still required, leading to initial buttons still
	-- having a zero count showing.  Later actionbar events will cause updates on
	-- *some* buttons (specifically, the ones that are paged or have other changing-state
	-- type stuff going on) and they will be corrected by the time the loading bar
	-- clears.  The remaining buttons are never updated though.
	--
	-- Workaround is to track all buttons which could be affected.  When a given
	-- button+action combo is updated, check any other buttons with the same action.
	if not action2button[action] then
		action2button[action] = {}
	end
	action2button[action][button.id] = button

	local hasReagent, hasCharges
	local oldCVar = GetCVar("UberTooltips")
	SetCVar("UberTooltips", "1")

	tooltip:SetOwner(UIParent, "ANCHOR_NONE")
	tooltip:SetAction(action)

	for line = 1, tooltip:NumLines() do
		local lineText = _G["BT4ThereIsNoCostTooltipTextLeft"..line]:GetText()
		if lineText then
--safeprint(lineText)
			if lineText:match("^"..SPELL_REAGENTS) then
				hasReagent = true
				break
			elseif lineText:match(CHARGE) or lineText:match(CHARGES) then
				hasCharges = true
				break
			end
		end
	end

	SetCVar("UberTooltips", oldCVar)
	tooltip:Hide()
--safeprint(hasReagent, hasCharges)

	-- This is probably already stored in the button already, either a field or
	-- an attribute, but fuck if I can figure out where.
	local actionType, id = GetActionInfo(action)

	if hasReagent or hasCharges
	   or (isConStack and (actionType == 'item' or GetMacroItem(id)))
	then
		-- normal case
		--safeprint("normal case, returning false")
		return false
	else
		-- special case
		--safeprint("special case, returning true")
		return true
	end
end


--[[
-- This doesn't actually help us, as not enough information is present yet
-- (to wit, _state_type is 'empty').
LAB.RegisterCallback("BT4-ThereIsNoCost", "OnButtonCreated", function(event,button)
	--safeprint("CREATED", button._state_type, button._state_action)
end)
]]


-- Because this includes things like action bar paging, it's called often.  Use
-- stored information about the *action* to determine whether the *button*
-- should be altered (they are not the same thing what with paging and whatnot).
local function do_update (event, button)
	if button._state_type ~= 'action' then return end  -- see TODO
	checked_actions[button._state_action] = checked_actions[button._state_action] or lookup_action(button)
	if checked_actions[button._state_action] then
		for id,but in pairs(action2button[button._state_action]) do
			--safeprint("checking text for", id, but, button, but.id, but._state_type)
			if but._state_type == 'action' then
				--safeprint("clearing text for", but.id)
				but.count:SetText("")
			end
		end
	end
end
LAB.RegisterCallback("BT4-ThereIsNoCost", "OnButtonUpdate", do_update)


-- Reset the cache if glyphs happen.  No data on which events mean what.  Could
-- probably make some guesses based on the names, but it's a rare operation, so
-- just do it for all events.
eventframe:RegisterEvent("GLYPH_ADDED")
eventframe:RegisterEvent("GLYPH_DISABLED")
eventframe:RegisterEvent("GLYPH_ENABLED")
eventframe:RegisterEvent("GLYPH_REMOVED")
eventframe:RegisterEvent("GLYPH_UPDATED")
eventframe:SetScript("OnEvent", function(frame,event,...)
--safeprint("===== TINC TRIGGER", event, ...)
	checked_actions = {}
	action2button = {}
	for button in next, LAB:GetAllButtons() do
		if button._state_type == 'action' then
			do_update (event, button)
		end
	end
end)
eventframe:Show()

-- vim: noet
